<?php
/**
 * @file
 */

class HostingDistributionMakefileUpdate {

  // A label for this type of update.
  public static $label = 'makefile';

  // Fields that should be formatted in the Aegir style on distribution nodes.
  public static $info_fields = array('field_drush_makefile');

  // Fields that should be made read-only on distribution node forms.
  public static $readonly_fields = array('field_drush_makefile_contents');

  // Other fields managed by this extention.
  public static $other_fields = array();

  // The HostingDistribution object that is being updated.
  public $distribution = FALSE;

  // A HostingDistributionLog object.
  private $log = FALSE;

  public function __construct($distribution = FALSE) {
    if ($distribution) {
      $this->setDistribution($distribution);
      $this->log = $this->distribution->log;
    }
  }

  public function setDistribution(HostingDistribution $distribution) {
    $this->distribution = $distribution;
  }

  public function update() {
    $this->log->info('Performing makefile plugin updates.');
    if ($this->logSetMakefileContents()) {
      $this->log->success('Performed makefile plugin updates.');
      return TRUE;
    }
    else {
      $this->log->warning('Failed to perform makefile plugin updates.');
      return FALSE;
    }
  }

  /**
   * A validate callback to ensure the Drush makefile is accessible.
   */
  public function validate(&$form, &$form_state) {
    $update_form = new HostingDistributionUpdateForm($form, $form_state);
    if ($makefile = $update_form->getFormStateValue('field_drush_makefile')) {
      if (!file_get_contents($makefile)) {
        return form_error($update_form->form['update_type_fieldset']['makefile_fieldset']['field_drush_makefile'], t('Cannot resolve Drush makefile. Please ensure that you have the correct path or URL.'));
      }
    }
  }

  /**
   * Return a hash of the current makefile.
   */
  public function calculateSignature() {
    return hash('md5', $this->getMakefileContents());
  }

  /**
   * Return a hash of the current Drush makefile.
   */
  private function getMakefile() {
    return $this->distribution->getFieldValue('field_drush_makefile');
  }

  /**
   * Fetch the contents of the specified Drush makefile.
   */
  private function getMakefileContents() {
    if ($contents = file_get_contents($this->getMakefile())) {
      return $contents;
    }
  }

  /**
   * Save the contents of the specified Drush makefile to the distribution node.
   */
  private function setMakefileContents() {
    return $this->distribution->setFieldValue('field_drush_makefile_contents', $this->getMakefileContents());
  }

  /**
   * Logging wrapper around setting makefile contents.
   */
  private function logSetMakefileContents() {
    $this->log->info(t('Setting new Drush makefile contents.'));
    $this->log->debug(t('New Drush makefile path: :path.', array(':path' => $this->getMakefile())));
    $this->log->debug(t("New Drush makefile contents: :contents.", array(':contents' => $this->getMakefileContents())));
    if ($this->setMakefileContents()) {
      $this->log->success(t('Set Drush makefile contents.'));
      return TRUE;
    }
    else {
      $this->log->warning(t('Failed to set Drush makefile contents.'));
      return FALSE;
    }
  }

  /**
   * Alter a new platform before it is saved.
   */
  public function alterPlatform(HostingDistributionPlatform &$platform) {
    return $platform->node->frommakefile = array('makefile' => $this->getMakefile());
  }

  /**
   * Return a hosting queue definition.
   */
  public function getQueue() {
    return array(
      'distribution_makefile_update' => array(
        'type' => 'batch',
        'name' => t('Drush makefile distribution update queue'),
        'description' => t('Check makefile-based distributions for updates.'),
        'total_items' => count($this->getDistributions()),
        'frequency' => strtotime("1 hour", 0),
        'singular' => t('distribution'),
        'plural' => t('distributions'),
      )
    );
  }

  /**
   * Return a list of NIDs for distributions that use a Drush Makefile.
   */
  static public function getDistributions() {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', 'distribution')
      ->propertyCondition('status', NODE_PUBLISHED)
      ->fieldCondition('field_drush_makefile', 'value', 'NULL', '!=');

    $result = $query->execute();
    if (isset($result['node'])) {
      return array_keys($result['node']);
    }
    else {
      return array();
    }
  }

  public function getLabel() {
    return self::$label;
  }

}
